#!/usr/bin/perl
# Guesses the paradigm for unknown words, using rules
# from a file.
#   guess_pdgm.pl  pdgm.rule < pdict  > pdict.2

@vowels = ('a','A','i','I','u','U','e','E','o','O','M');

# Read the rule file with the following format:
#   adj_A	kAlA
#   adj_M	kAlA
#   nf_A	BASA
#   nf_I	kudZI
#   nf_M	sIrIM OTHERS: WAM sUM 
#   nf_O	rO
#   nm_A	muMdA
# Ignore lines with OTHERS:
open (RULES, $ARGV[0]) || 
  die "Unable to open rules for guessing paradigm file $ARGV[0]\n";
while (<RULES>) {
  chop;
  if (/\sOTHERS:\s/) { next; }
  ($pat, $pdgm) = split (/\s+/);
  if ($pdm_for_ending{$pat} ne '') { 
    print STDERR 
       "Error in rules file $ARGV[0]. Skipping repeated pattern: $_\n";
  }
  else { $pdm_for_ending{$pat} = $pdgm; }
}

# Now read the dictionary and write guesses for paradigms.
while (<STDIN>) {
  if (/^"([^"]+)","([^"]+)","([^"]+)","([^"]+)","([^"]+)"$/) {
    $head_wrd = $1;    $target_wrd = $4;
    $pdm_type = $2;    $cat = $3;
    if ($pdm_type eq 'UNKN') { 
      if ($head_wrd =~ /(.)$/) {
        $test_pat = $cat . '_' . $1;
        $guess_pdm = $pdm_for_ending{$test_pat};
        if ($guess_pdm) {
          print "\"$head_wrd\",\"$guess_pdm\",\"$cat\",\"$target_wrd\"\n";
          $cnt_guess++;
        }
        else { 
          print;
          $cnt_not_guess++;
          #print STDERR "NOGUESS:$_";
        }
      }
      else { print STDERR "Head word is nil: $_"; }
    }
    else { print; }  # Not unknown paradigm. Write as is.
  }
  else { print STDERR "Invalid format for line $.:$_"; }
}
print STDERR 'guess_pdgm.pl: Count of UNKN pdgms: ',
  "Guessed: $cnt_guess, Not guessed: $cnt_not_guess,\n";
